{% extends "flower/base.html" %}

{% block container %}
    <div class="span12">
        <p>
            {% if message %}
                {{ message }}
            {% else %}
                Error, page not found
            {% endif %}
        </p>
    {% include 'flower/traceback.html' %}
    </div>
{% endblock %}
